@@ -205,6 +205,26 @@ class Desktop:
             url=data.get("url")
         )
     
+    def get_vnc_url(self) -> str:
+        """
+        Get VNC URL (convenience method).
+        
+        This is a convenience method that returns just the URL string
+        instead of the full VNCInfo object.
+        
+        Returns:
+            VNC URL string
+        
+        Raises:
+            DesktopNotAvailableError: If desktop not available
+        
+        Example:
+            >>> url = sandbox.desktop.get_vnc_url()
+            >>> print(f"Connect to: {url}")
+        """
+        vnc_info = self.get_vnc_status()
+        return vnc_info.url or ""
+    
     # =============================================================================
     # MOUSE CONTROL
     # =============================================================================
@@ -860,6 +880,31 @@ class Desktop:
             operation="resize window"
         )
     
+    def minimize_window(self, window_id: str) -> None:
+        """
+        Minimize window.
+        
+        Args:
+            window_id: Window ID from get_windows()
+        
+        Raises:
+            DesktopNotAvailableError: If desktop not available
+        
+        Example:
+            >>> windows = sandbox.desktop.get_windows()
+            >>> if windows:
+            ...     sandbox.desktop.minimize_window(windows[0].id)
+        """
+        self._check_availability()
+        
+        logger.debug(f"Minimize window {window_id}")
+        
+        self._client.post(
+            "/desktop/windows/minimize",
+            json={"window_id": window_id},
+            operation="minimize window"
+        )
+    
     # =============================================================================
     # DISPLAY
     # =============================================================================
