@@ -1,6 +1,8 @@
 """Main Sandbox class - E2B inspired pattern."""
 
 from typing import Optional, List, Iterator, Dict, Any
+from datetime import datetime, timedelta
+from dataclasses import dataclass
 import logging
 
 # Public API models (enhanced with generated models + convenience)
@@ -27,6 +29,17 @@ from .terminal import Terminal
 logger = logging.getLogger(__name__)
 
 
+@dataclass
+class TokenData:
+    """JWT token storage."""
+    token: str
+    expires_at: datetime
+
+
+# Global token cache (shared across all Sandbox instances)
+_token_cache: Dict[str, TokenData] = {}
+
+
 class Sandbox:
     """
     Bunnyshell Sandbox - lightweight VM management.
@@ -291,8 +304,17 @@ class Sandbox:
         if self._agent_client is None:
             info = self.get_info()
             agent_url = info.public_host.rstrip('/')
+            
+            # Ensure JWT token is valid
+            self._ensure_valid_token()
+            
+            # Get JWT token for agent authentication
+            jwt_token = _token_cache.get(self.sandbox_id)
+            jwt_token_str = jwt_token.token if jwt_token else None
+            
             self._agent_client = AgentHTTPClient(
                 agent_url=agent_url,
+                jwt_token=jwt_token_str,
                 timeout=60,  # Default 60s for agent operations
                 max_retries=3
             )
@@ -306,6 +328,67 @@ class Sandbox:
             self._ws_client = WebSocketClient(agent_url)
             logger.debug(f"WebSocket client initialized: {agent_url}")
     
+    def refresh_token(self) -> None:
+        """
+        Refresh JWT token for agent authentication.
+        Called automatically when token is about to expire (<1 hour left).
+        """
+        response = self._client.post(f"/v1/sandboxes/{self.sandbox_id}/token/refresh")
+        
+        if "auth_token" in response and "token_expires_at" in response:
+            _token_cache[self.sandbox_id] = TokenData(
+                token=response["auth_token"],
+                expires_at=datetime.fromisoformat(response["token_expires_at"].replace("Z", "+00:00"))
+            )
+            
+            # Update agent client's JWT token if already initialized
+            if self._agent_client is not None:
+                self._agent_client.update_jwt_token(response["auth_token"])
+    
+    def _ensure_valid_token(self) -> None:
+        """
+        Ensure JWT token is valid (not expired or expiring soon).
+        Auto-refreshes if less than 1 hour remaining.
+        """
+        token_data = _token_cache.get(self.sandbox_id)
+        
+        if token_data is None:
+            # No token yet, try to refresh
+            try:
+                self.refresh_token()
+            except Exception:
+                # Token might not be available yet (e.g., old sandbox)
+                pass
+            return
+        
+        # Check if token expires soon (< 1 hour)
+        now = datetime.now(token_data.expires_at.tzinfo)
+        hours_left = (token_data.expires_at - now).total_seconds() / 3600
+        
+        if hours_left < 1:
+            # Refresh token
+            self.refresh_token()
+    
+    def get_token(self) -> str:
+        """
+        Get current JWT token (for advanced use cases).
+        Automatically refreshes if needed.
+        
+        Returns:
+            JWT token string
+        
+        Raises:
+            BunnyshellError: If no token available
+        """
+        self._ensure_valid_token()
+        
+        token_data = _token_cache.get(self.sandbox_id)
+        if token_data is None:
+            from .errors import BunnyshellError
+            raise BunnyshellError('No JWT token available for sandbox')
+        
+        return token_data.token
+    
     # =============================================================================
     # CLASS METHODS (Static - for creating/listing sandboxes)
     # =============================================================================
@@ -313,11 +396,9 @@ class Sandbox:
     @classmethod
     def create(
         cls,
-        template: str,
+        template: Optional[str] = None,
         *,
-        vcpu: int = 2,
-        memory_mb: int = 2048,
-        disk_gb: Optional[int] = None,
+        template_id: Optional[str] = None,
         region: Optional[str] = None,
         timeout: int = 300,  # 5 minutes default
         env_vars: Optional[Dict[str, str]] = None,
@@ -325,13 +406,14 @@ class Sandbox:
         base_url: str = "https://api.hopx.dev",
     ) -> "Sandbox":
         """
-        Create a new sandbox.
+        Create a new sandbox from a template.
+        
+        Resources (vcpu, memory, disk) are ALWAYS loaded from the template.
+        You cannot specify custom resources - create a template first with desired resources.
         
         Args:
-            template: Template name (e.g., "code-interpreter", "nodejs", "python")
-            vcpu: Number of vCPUs (default: 2)
-            memory_mb: Memory in MB (default: 2048)
-            disk_gb: Disk size in GB (optional, uses template default)
+            template: Template name (e.g., "my-python-template")
+            template_id: Template ID (alternative to template name)
             region: Preferred region (optional, auto-selected if not specified)
             timeout: Sandbox timeout in seconds (default: 300 = 5 minutes)
             env_vars: Environment variables to set in the sandbox (optional)
@@ -346,40 +428,49 @@ class Sandbox:
             ResourceLimitError: Insufficient resources
             APIError: API request failed
         
-        Example:
-            >>> sandbox = Sandbox.create(template="code-interpreter")
+        Examples:
+            >>> # Create from template ID
+            >>> sandbox = Sandbox.create(template_id="291")
             >>> print(sandbox.get_info().public_host)
             
-            >>> # With custom resources and environment variables
+            >>> # Create from template name
             >>> sandbox = Sandbox.create(
-            ...     template="nodejs",
-            ...     vcpu=4,
-            ...     memory_mb=4096,
-            ...     timeout=600,
-            ...     env_vars={
-            ...         "API_KEY": "sk-prod-xyz",
-            ...         "DATABASE_URL": "postgres://localhost/db",
-            ...         "NODE_ENV": "production"
-            ...     }
+            ...     template="my-python-template",
+            ...     env_vars={"DEBUG": "true"}
             ... )
         """
         # Create HTTP client
         client = HTTPClient(api_key=api_key, base_url=base_url)
         
-        # Build request payload
-        data = remove_none_values({
-            "template_name": template,
-            "vcpu": vcpu,
-            "memory_mb": memory_mb,
-            "disk_gb": disk_gb,
-            "region": region,
-            "env_vars": env_vars,
-        })
+        # Validate parameters
+        if template_id:
+            # Create from template ID (resources from template)
+            data = remove_none_values({
+                "template_id": template_id,
+                "region": region,
+                "env_vars": env_vars,
+            })
+        elif template:
+            # Create from template name (resources from template)
+            data = remove_none_values({
+                "template_name": template,
+                "region": region,
+                "env_vars": env_vars,
+            })
+        else:
+            raise ValueError("Either 'template' or 'template_id' must be provided")
         
         # Create sandbox via API
         response = client.post("/v1/sandboxes", json=data)
         sandbox_id = response["id"]
         
+        # ⚠️ NEW: Store JWT token from create response
+        if "auth_token" in response and "token_expires_at" in response:
+            _token_cache[sandbox_id] = TokenData(
+                token=response["auth_token"],
+                expires_at=datetime.fromisoformat(response["token_expires_at"].replace("Z", "+00:00"))
+            )
+        
         # Return Sandbox instance
         return cls(
             sandbox_id=sandbox_id,
@@ -398,6 +489,12 @@ class Sandbox:
         """
         Connect to an existing sandbox.
         
+        NEW JWT Behavior:
+        - If VM is paused → resumes it and refreshes JWT token
+        - If VM is stopped → raises error (cannot connect to stopped VM)
+        - If VM is running/active → refreshes JWT token
+        - Stores JWT token for agent authentication
+        
         Args:
             sandbox_id: Sandbox ID
             api_key: API key (or use BUNNYSHELL_API_KEY env var)
@@ -408,6 +505,7 @@ class Sandbox:
         
         Raises:
             NotFoundError: Sandbox not found
+            BunnyshellError: If sandbox is stopped or in invalid state
         
         Example:
             >>> sandbox = Sandbox.connect("1761048129dsaqav4n")
@@ -421,8 +519,30 @@ class Sandbox:
             base_url=base_url,
         )
         
-        # Verify it exists by fetching info
-        instance.get_info()
+        # Get current VM status
+        info = instance.get_info()
+        
+        # Handle different VM states
+        if info.status == "stopped":
+            from .errors import BunnyshellError
+            raise BunnyshellError(
+                f"Cannot connect to stopped sandbox {sandbox_id}. "
+                "Use sandbox.start() to start it first, or create a new sandbox."
+            )
+        
+        if info.status == "paused":
+            # Resume paused VM
+            instance.resume()
+        
+        if info.status not in ("running", "paused"):
+            from .errors import BunnyshellError
+            raise BunnyshellError(
+                f"Cannot connect to sandbox {sandbox_id} with status '{info.status}'. "
+                "Expected 'running' or 'paused'."
+            )
+        
+        # Refresh JWT token for agent authentication
+        instance.refresh_token()
         
         return instance
     
